import {
  min, 
  max,
  select, 
  selectAll,
  scaleBand,
  scaleLinear,
  scaleOrdinal,
  axisBottom,
  axisLeft,
  transition,
} from 'd3';
import { colors, getScale } from './utils';
// import { legend } from './legend'

export const barplot = () => {
  // Globals
  let options = {};
  let data = [];

  // Graph
  const my = (selection) => {
    // Dimensions
    const width = options.width;
    const height = options.height;
    const margin = options.margin;
    
    const group = data.map(d => d[options.xKey]);
    const subgroup = options.yKey;
    
		// Transition
    const t = transition().duration(800);

    // Draw X-axis
    const labels = []
    const xScale = getScale('band')
      .domain(group)
      .padding(0.2)
      .range([options.margin.left, width - margin.right]);

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', '.x-axis')
      .attr(
        'transform',
        `translate(${0}, ${
          height - margin.bottom
        })`
      )
    	.transition(t)
      .call(axisBottom(xScale));
    
    // Draw Y-axis
    const yValues = data.map(d =>
  		subgroup.map(key => d[key])
		).flat();
    
    const yScale = getScale()
      .domain([0, max(yValues) + 50])
      .range([
        height - margin.bottom,
        margin.top,
      ]);
    
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', '.y-axis')
      .attr(
        'transform',
        `translate(${margin.left}, ${0})`
      )
    	.transition(t)
      .call(axisLeft(yScale));
    
    // X-SubGroup
    const xSubscale = scaleBand()
      .domain(subgroup)
      .padding(0.05)
      .range([0, xScale.bandwidth()]);

    // Colors
    const colorScale = scaleOrdinal()
    	.domain(subgroup)
    	.range(colors);
    
    // Process Data    
    const marks = data.map((d, i) => ({
      id: i,
    	group: d.label,
      data: subgroup.map(key => ({
        key: key, 
        value: d[key], 
        color: colorScale(key),
        display: true,
      }))
    }));
    
    // Draw Bars
    createOrUpdateChart(marks);
    
    // Create Legend
    const legendKeys = marks[0].data;
    createLegend(legendKeys);
    
    // Other Function
    function createOrUpdateChart(data) {
    	const bars = selection
        .selectAll('rect')
        .data(data, d => d.id) // Key function
        .join('g')
          .attr('transform', d => `translate(${xScale(d.group)}, ${0})`)
        .selectAll('rect')
        .data((d) => d.data)
        .join(
          (enter) => {
            enter
              .append('rect')
                .attr('x', (d) => xSubscale(d.key))
                .attr('width', (d) => xSubscale.bandwidth())
                .attr('fill', (d) => d.color)
                .attr('y', (d) => yScale(0))
                .attr('height', 0)
                .call((enter) => {
                  enter
                    // .transition(t)
                    .attr('y', (d) => yScale(d.value))
                    .attr('height', (d) => height - yScale(d.value) - margin.bottom)
                    // .delay((d, i) => i * 100)
                  }
                )
          },
          (update) => 
            update.call((update) => 
              update
                // .transition(t)
                // .delay((d, i) => i * 10)
            ),
          (exit) => exit.remove()
        );
    }
    
    function createLegend(data) {
      const legendRectSize = 18, legendSpacing = 4;
      const legend = selection
        .selectAll('.legend')
        .data(data)
        .join('g')
        .attr('class', 'legend')
        .attr('transform', (d, i) => {
          let height = legendRectSize + legendSpacing;
          let offset =  height * 4 / 2;
          let horz = -2 * legendRectSize;
          let vert = i * height - offset + 70; 
          return `translate(${width - margin.right - 100}, ${vert})`
        })

      legend
        .append('rect')
        .attr('width', legendRectSize)
        .attr('height', legendRectSize)
        .style('fill', d => d.color)
        .style('stroke', d => d.color)

      legend
        .append('text')
        .attr('x', legendRectSize + legendSpacing)
        .attr('y', legendRectSize - legendSpacing)
        .text(d => d.key)
        .style('cursor', 'pointer')
        .style('user-select', 'none')
        .on('click', toggleBars);
    }
    
    function toggleBars(e, d) {
  		const isStrikethrough = d3.select(this).classed("strikethrough");
  		d3.select(this).classed("strikethrough", !isStrikethrough);
      
      if(!isStrikethrough) {
        const newMarks = [...marks.map(m => ({...m, data: m.data.filter(dt => dt.key !== d.key)}))];
        createOrUpdateChart(newMarks);
        console.log(newMarks)
      } else {
        createOrUpdateChart(marks)
        console.log(marks)
      }
		}
  };
  
  // Getters & Setters
  my.options = function (_) {
    return arguments.length
      ? ((options = _), my)
      : options;
  };
  my.data = function (_) {
    return arguments.length
      ? ((data = _), my)
      : data;
  };

  return my;
};
